/***************** (c) 2006 BTC Korporacja ***********************************
                     http://www kamami.com
PROJECT  : ADC_LCD
BOARD    : ZL1ST7 (with ST7FLITE29 in socket U6)
COMPILER : COSMIC (IDE ST7VD)

******************************************************************************

    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
    BTC KORPORACJA SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT 
    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING 
    FROM USE OF THIS SOFTWARE.

******************************************************************************

DESCRIPTION :
    Example of AD conversion. Result of 10 bit conversion is displayed on LCD
    display (HD44780 compatible) in 4-bit mode in hex format: 0xDDD.
    Jumpers settings on ZL1ST7:
        JP3  - 2-3 - crystal 16MHz
        JP4  - 2-3 - crystal 16MHz
        JP8  - 1-2 - signal from photoresistor (R9)
               2-3 - signal from potentiometer P1
        JP9  - 1-2 - connect analog signal to PB4 (AIN4)

    LCD - MCU (RW=GND)
     D4 - PA0
     D5 - PA1
     D6 - PA2
     D7 - PA3
     E  - PA7
     RS - PA4

******************************************************************************/
                   
#include <io7flit2.h> 

void delay (unsigned int del_time);                // simple delay function
void LCDDelay(int del);                            // delay function for LCD
void LCDText(char *T);                
void LCDTextXY(char x, char y, char *T);
void LCDSendByte(unsigned char data_lcd,unsigned char cmd);
void LCDInit(void);
void LCDHex(unsigned char hex);
unsigned int get_adc(void);

const LCD_delay = 150;
const unsigned char E  = 0x80;           // PA7
const unsigned char RS = 0x10;           // PA4
const unsigned char dat_lcd = 0x10; 	 // send data to LCD
const unsigned char cmd_lcd = 0x00; 	 // send command to LCD


void main(void)
{
  unsigned int val;
  
  PADDR = 0xFF;	                // PORTA: bits 0..7 - outputs
  PAOR = 0xFF;		        // PORTA: push-pull
 
  LCDInit();                    // LCD initialization
  
  LCDText(" ##  ZL1ST7  ##");   
  delay(10);   
  while(1)
  {
    LCDTextXY(0,1, "AIN4 = 0x");
    val = get_adc();
    LCDHex((val >> 8) & 0x0F);         // 1 hex digit
    LCDHex((val >> 4) & 0x0F);         // 2 hex digit
    LCDHex(val & 0x0F);                // 3 hex digit
    delay(10);
  }                     
}   
/////////////////////////////////////////////////////////////

/*-----------------------------------------------------------------------------
FUNCTION NAME : delay
INPUT/OUTPUT  : del_time
DESCRIPTION   : simple delay function
-----------------------------------------------------------------------------*/ 
void delay (unsigned int del_time) 
{
  unsigned int temp;
                     
  while( del_time-- )
    for(temp=0; temp<5000; temp++)
      ;
} 


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDDelay
INPUT/OUTPUT  : del
DESCRIPTION   : simple delay function for LCD functions
-----------------------------------------------------------------------------*/ 
void LCDDelay(volatile int del)
{
 for(;del>0;del--)
   ;
} 

/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDGotoXY
INPUT/OUTPUT  : x,y - position on LCD display ( (0,0) = left, up )
DESCRIPTION   : Set actual position to (x,y) 
-----------------------------------------------------------------------------*/ 
void LCDGotoXY(char x, char y)
{
  switch (y)
  {
    case 0:
        x += 0x80;
        break;
    case 1:
        x += 0xC0;
        break;
    case 2:
        x += 0x94;
        break;
    case 3:
        x += 0xD4;
        break;
  }
  LCDSendByte( x , cmd_lcd );
}


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDText
INPUT/OUTPUT  : T - string to be displayed
DESCRIPTION   : Display string T on LCD at actual position 
-----------------------------------------------------------------------------*/ 
void LCDText(char *T)
{
  while (*T)
    LCDSendByte(*T++,dat_lcd);         // display character
}


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDTextXY
INPUT/OUTPUT  : x,y - position on LCD display ( (0,0) = left, up )
                T   - string to be displayed
DESCRIPTION   : Display string T on LCD at position (x,y) 
-----------------------------------------------------------------------------*/ 
void LCDTextXY(char x, char y, char *T)
{
  while (*T)
  {
    LCDGotoXY(x, y);                   // set position
    LCDSendByte(*T,dat_lcd);           // display first character
    x++;                               // next position 
    T++;                               // next character
    if (x > 19)                        // x>19 -> go to next line
    {
      x = 0; 
      y++;
    }
  }  // while
}				   


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDSendByte
INPUT/OUTPUT  : data_lcd - data byte
                cmd      - = dat_lcd (0x10) send data byte to LCD
                           = cmd_lcd (0x00) send command byte to LCD
DESCRIPTION   : LCD initialization
-----------------------------------------------------------------------------*/ 
void LCDSendByte(unsigned char data_lcd,unsigned char cmd)
{
  PADR = ((data_lcd >> 4) & ~(E)) | cmd;    //RS=cmd, E=0 
  LCDDelay( LCD_delay );
  PADR = (data_lcd >> 4) | E | cmd;         //RS=cmd, E=1   
  LCDDelay( LCD_delay );
  PADR = ((data_lcd >> 4) & ~(E)) | cmd;    //RS=cmd, E=0 
  LCDDelay( LCD_delay );
    
  PADR = (data_lcd & ~(E)) | cmd;           //RS=cmd, E=0
  LCDDelay( LCD_delay );
  PADR = (data_lcd) | E | cmd;              //RS=cmd, E=1
  LCDDelay( LCD_delay );
  PADR = (data_lcd & ~(E)) | cmd;           //RS=cmd, E=0  
  LCDDelay( LCD_delay ); 
} 


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDInit
INPUT/OUTPUT  : none
DESCRIPTION   : LCD initialization
-----------------------------------------------------------------------------*/ 
void LCDInit(void)
{
  LCDDelay(10000);
  
  PADR = 0x03;         //D5=1, D4=1
  PADR = 0x83;         //E=1
  PADR = 0x03;         //E=0
  LCDDelay( 1000 );
  
  PADR = 0x03;         //D5=1, D4=1
  PADR = 0x83;         //E=1
  PADR = 0x03;         //E=0
  LCDDelay( 1000 );
  
  PADR = 0x03;         //D5=1, D4=1
  PADR = 0x83;         //E=1
  PADR = 0x03;         //E=0
  LCDDelay( 1000 );
  
  PADR = 0x02;         //D5=1, D4=0
  PADR = 0x82;         //E=1
  PADR = 0x02;         //E=0
  LCDDelay(1000);

  LCDSendByte( 0x28, cmd_lcd);         // 2 lines, 4 bits 
  LCDDelay(1000);                                          
  LCDSendByte( 0x08, cmd_lcd);         // LCD off                           
  LCDDelay(1000);                
  LCDSendByte( 0x06, cmd_lcd);         // cursor inc
  LCDDelay(1000);
  LCDSendByte( 0x01, cmd_lcd);         // clear LCD, cursor home
  LCDDelay(1000);
  LCDSendByte( 0x0c, cmd_lcd);         // LCD on
  LCDDelay(5000);
}   


/*-----------------------------------------------------------------------------
FUNCTION NAME : LCDHex
INPUT/OUTPUT  : hex - value (0...15) to display on LCD
DESCRIPTION   : write hex digit on LCD
-----------------------------------------------------------------------------*/ 
void LCDHex(unsigned char hex)
{
  if (hex > 9) LCDSendByte( 'A' + (hex - 10), dat_lcd );
  else         LCDSendByte( '0' +  hex, dat_lcd );
}


/*-----------------------------------------------------------------------------
FUNCTION NAME : get_adc
INPUT/OUTPUT  : returns 10 bit adc result
DESCRIPTION   : AD conversion
-----------------------------------------------------------------------------*/ 
unsigned int get_adc(void)
{
  
  ADCDRL = 0x08;                       // cklock for ADC - SLOW mode (fcpu/4)
  ADCCSR = ADCCSR | 0x24;              // AIN4 as analog input, ADC on
  while(!(ADCCSR & 0x80))              // wait for end of conversion (EOC)
    ;
  return (ADCDRL & 0x03) | (ADCDRH << 2);  // return 10 bit result of conversion
}
